/* Project: Robotik-Praktikum - "Rug Warrior"                                                                              */
/* File   : "DRIVE.ASM"                                                                                                    */
/* Author : Lars Ferner / Robin Wissbrock                                                                                                    */
/* Date   : 96-06-12 / 95-12-08                                                                                            */
/* Notes  : Assembler-Modul fuer Abfrage der Rad-Encoder und Stoppen der Motoren zur Vermeidung von "Huepfern"             */


TFLG1 equ $1023                                            /* Constants                                                    */


	org MAIN_START                                          /* Set program counter to 'MAIN_START'                          */


variable_left_clicks:                                      /* Create 'left_clicks', a variable accessible by IC            */
	fdb 0                                                   /* Fill double byte (16 bits) -> "left_clicks = 0"              */

variable_test_clicks:                                      /* Create 'right_clicks', a variable accessible by IC           */
	fdb 0


subroutine_initialize_module:                              /* This routine runs whenever a reset occurs                    */
	ldd #IC3_interrupt_handler                              /* Get 16-bit address of interrupt handler                      */
	std $BFEA                                               /* Store in IC3 interrupt vector location                       */
	cli                                                     /* Enable interrupts globally                                   */
	rts                                                     /* Return from initialzie_module subroutine                     */

IC3_interrupt_handler:
	ldd variable_left_clicks                                /* Get the value in 'left_clicks'                               */
	addd #1                                                 /* Add one more encoder count                                   */
	std variable_left_clicks                                /* Store back to 'left_clicks'                                  */
	ldaa #%00000001                                         /* Clear the IC3 flag by writing a one                          */
	staa TFLG1                                              /* Store in TFLG1 to clear IC3 flag                             */
	rti                                                     /* Return from interrupt                                        */

subroutine_test_proc:                                      /* Test-Prozedur                                                */
	ldaa $1027
	staa variable_test_clicks
	ldaa #0
	staa $1027
	rts


/* Stoppen der Motoren, falls neuer PWM-Wert kleiner als aktueller Wert des Zaehlers vermeidet Ausreisser nach oben beim   */
/* Abbremsen (wird von motor() benutzt, aus Zeitgruenden in Assembler realisiert.                                          */
/* Eine kleine Unsauberkeit existiert noch:                                                                                */
/* mit 0,03 % Wahrscheinlichkeit kommt es beim Aendern des PWM-Werts zu einem Aussetzer, dies passiert, wenn zwischen der  */
/* Abfrage des Timers und dem Stopp des Motors der Timer ablaeuft und deswegen in dieser Periode nicht mehr gestartet wird.*/
/* Dieses "feature" hat aber noch nicht zu wahrnehmbarem Fehlverhalten gefuehrt.                                           */

subroutine_ClearSignal_0:
/* linker Motor - Trennung in zwei Routinen, da einer Assemblerroutine nur ein 16-bit-Wert uebergeben werden kann          */
	sei
	ldx #$1000                                              /* Basisadresse fuer das untenstehende 'bset'. Der Umweg ueber  */
																			  /* das x-Register ist noetig, weil bei 'bset' nur 8-bit-Adressen*/
																			  /* verwendet werden duerfen.                                    */
																			  /* ACHTUNG: der Assembler meckert NICHT bei der Verwendung einer*/
																			  /* 16-bit Adresse, im icb-File steht aber dann nur das LSB      */
																			  /* (boese Falle)!                                               */
	cpd $100e                                               /* neu zu programmierender Wert (in d) mit Zaehlerstand nicht   */
	bhi nothing_0                                           /* im kritischen Bereich -> 'rts'                               */
	bset $0b, #64, x                                        /* Motor stoppen                                                */

nothing_0:
	cli
	rts

subroutine_ClearSignal_1:
/* rechter Motor - analog                                                                                                  */
	sei
	ldx #$1000

	cpd $100e
	bhi nothing_1
	bset $0b, #32, x

nothing_1:
	cli
	rts

