/* Project: Rug Warrior                                                                                                    */
/* File   : "LIB_RW11.C"                                                                                                   */
/* Author : Lars Ferner, Uni Kaiserslautern                                                                                */
/* Date   : 96-09-16 / 95-12-07 / 29-07-97                                                                                 */
/* Notes  : Library for Rug Warrior from an earlier library by jsr, fgm and jlj                                            */
/*          ACHTUNG: die Funktionen zur Ansteuerung der Motoren befindet sich in "DRIVE(*).C"                              */


/* Global constants                                                                                                        */

int photo_right = 0;                                       /* Indices for accessing sensors connected to the A/D           */
int photo_left = 1;                                        /* converter, e.g. to read value of right photo cells use       */
int microphone = 2;                                        /* analog( photo_right )                                        */
int pyro = 5;

int FALSE = 0, TRUE = 1;
float M_PI = 3.1415926536, M_2PI = 6.2831853072, M_PI2 = 1.5707963268;
int LEFT = 0, RIGHT = 1;
int OFF = 0, ON = 1;


/* Global variables                                                                                                        */

persistent int mode;
/* persistent float test[100]; */

long timer[5] = { 0L, 0L, 0L, 0L, 0L };                    /* Timers                                                       */

int rls_threshold = 6;


/* Functions (definitions)                                                                                                 */

void reset_system_time( void )
{
	pokeword( 0x14, 0 );                                    /* 0x14: time in milliseconds                              */
	pokeword( 0x12, 0 );
}

float seconds( void )
/* returns valid time from 0 to 32,767 minutes (approx)                                                                    */
{
	return ( (float) mseconds() / 1000.0 );
}

void msleep( long msec )
{
	/* Local variables                                                                                                 */

	long end_time = mseconds() + msec;


	while ( 1 )
	{
		long done = mseconds() - end_time;                   /* if the following test doesn't execute at least once a        */
		if ( ( done >= 0L ) && ( done <= 1000L ) )           /* second, msleep may not halt                                  */
			break;
	}
}

void sleep( float seconds )
{
	msleep( (long) ( seconds * 1000.0 ) );
}

void timer_start( int i, long timeout )
{
	timer[i] = mseconds() + timeout;
}

long timer_time( int i )
{
	return ( timer[i] - mseconds() );
}

void digital_out( int port, int value )
/* Set port of expansion board to value                                                                                         */
{
	if      ( port == 0 ) port = 0;
	else if ( port == 1 ) port = 4096;
	else if ( port == 2 ) port = 2048;
	else if ( port == 3 ) port = 6144;
	else if ( port == 4 ) port = 1024;
	else if ( port == 5 ) port = 5120;
	else if ( port == 6 ) port = 3072;
	else if ( port == 7 ) port = 7168;
	else                  port = -1;

	if ( ( port > -1 ) && ( ( value == 0 ) || ( value == 1 ) ) )
		peek( 16384 + port + 512 * value );
	else
		printf( "Parameter error: digital_out" );
}

int digital( int port )
/* Return 1 bit from PA1 or PA2 (or PA0)                                                                                        */
/* Digital ports 1 and 2 are unassigned                                                                                         */
{
	return ( 1 & ( peek( 0x1000 ) >> ( port & 3 ) ) );
}

int analog( int port )
/* Rug Warrior's analog ports 6 and 7 are unassigned                                                                            */
{
	int result;
	
        hog_processor();
	poke( 0x1039, 0b10000000 );
	poke( 0x1030, port );

	result = peek( 0x1031 );
	defer();

	return (result);
}

int bumper( void )
/* Return a 3-bit value representing which of the bumper switches are closed                                                    */
{
	/* Local variables                                                                                                      */

	int bmpr;


	bmpr = analog( 3 );                                     /* Switch closure:                                              */
	if      ( bmpr <  16 ) return ( 0b000 );                /*                                                              */
	else if ( bmpr <  38 ) return ( 0b001 );                /*   A                                                          */
	else if ( bmpr <  60 ) return ( 0b010 );                /*  B                                                           */
	else if ( bmpr <  80 ) return ( 0b011 );                /*  BA                                                          */
	else if ( bmpr < 101 ) return ( 0b100 );                /*  C                                                           */
	else if ( bmpr < 123 ) return ( 0b101 );                /* C A                                                          */
	else if ( bmpr < 146 ) return ( 0b110 );                /* CB                                                           */
	else                   return ( 0b111 );                /* CBA (mechanically impossible)                                */
}

int ir_detect( void )
/* Neue Version von Klaus und Michael, die etwas stabiler gegenueber Messfehlern ist!                                           */
/* ir_detect returns:                                                                                                           */
/*   0b00 => no reflection, 0b01 => reflection on right,                                                                        */
/*   0b10 => reflection on left, 0b11 => reflection on both sides                                                               */
{
	/* Local variables                                                                                                      */

	int val1, val2, val3;

		val1 = peek( 0x100A ) & 0b10000;                     /* IR Detector connected to PE4                                 */
		bit_set( 0x1008, 0b1000 );                           /* Turn on Left emitter, PD3                                    */
		msleep( 1L );                                        /* Wait 1 millisecond                                           */
		val2 = peek( 0x100A ) & 0b10000;                     /* Should be Low if signal detected                             */
		bit_clear( 0x1008, 0b1000 );                         /* Turn off Left emitter                                        */
		bit_set( 0x1008, 0b0100 );                           /* Turn on Right emitter, PD2                                   */
		msleep( 1L );                                        /* Wait 1 millisecond                                           */
		val3 = peek( 0x100A ) & 0b10000;
		bit_clear( 0x1008, 0b0100 );                         /* Turn emitter off                                             */
		msleep( 1L );				             /* For detection, detector must be high when emitter is off,    */
		val1 = val1 & peek( 0x100A );                        /* Should be High again                                         */
								     /* jede 0 (Hindernis obwohl kein Emmiter an ist!) merken!       */
	return ( ( val1 & ~val2 ) >> 3 ) | ( ( val1 & ~val3 ) >> 4 );   /* HI -> LOW                                                 */
}

int RLS( void )
/* Abfragen der Reflexlichtschranke: 0 = kein Objekt, 1 = unsicher, 2 = Objekt vorhanden                                             */
{
	/* Local variables  */
	
	int res = 0;
	

	digital_out( 6, OFF );
	if ( analog( 7 ) < rls_threshold )
		return ( 1 );
	
	digital_out( 6, ON );
	if ( analog( 7 ) < rls_threshold )
		res = 2;
	digital_out( 6, OFF );

	if ( analog( 7 ) < rls_threshold )
		return ( 1 );
	
	return ( res );
}

void set_beeper_pitch( float frequency )
{
	pokeword( 0x26, (int) ( 1E6 / frequency ) );
}

void beeper_on( void )
{
	bit_set( 0x1020, 0b00000001 );
	bit_set( 0x1022, 0b00001000 );
}

void beeper_off( void )
{
	bit_clear( 0x1022, 0b00001000 );
	bit_clear( 0x1020, 0b00000001 );
	pokeword( 0x26, 0 );
	bit_clear( 0x1000, 0b00001000 );                        /* turn power to spkr off                                       */
}

void tone( float frequency, int length )
/* 1/2 cycle delay in .5us goes in 0x26 and 0x27                                                                           */
{
	pokeword( 0x26, (int) ( 1E6 / frequency ) );
	bit_set( 0x1020, 0b00000001 );
	msleep( (long) length );
	bit_clear( 0x1020, 0b00000001 );

	pokeword( 0x26, 0 );                                    /* this is important to reduce # of interrupts when tone is off */

	bit_clear( 0x1000, 8 );
}

void beep( void )
{
	tone( 500.0, 300 );
}


int abs(int arg)						/* Absolute value of an integer */
{
	if (arg < 0)
      		return - arg;
    	else
		return arg;
}


float absf(float arg)						/* Absolute value of a float */
{
	if (arg > 0.0)
		return arg;
	else
		return -arg;
}


void hog_processor( void )
/* gives process that calls it 256 ticks (over 1/4 sec)                                                                    */
/* more to run before being swapped out                                                                                    */
/* call repeatedly to hog processor indefinitely                                                                           */
{
	poke( 0x0a, 0 );
}

