/* Project: Robotik-Praktikum - "Rug Warrior"                                                                              */
/* File   : "PSD.ASM"                                                                                                      */
/* Author : Robin Wissbrock, Uni Kaiserslautern                                                                            */
/* Date   : 96-03-25 / 96-02-28                                                                                            */
/* Notes  : Low-Level-Routinen fuer die Ansteuerung des PSD GP2D02 (s.a. "PSD.C").                                         */
/*          Benutzung von IC aus: StartPSD( 0 );                                                                           */
/*                                [70 ms warten];                                                                          */
/*                                ergebnis = GetPSD( 0 );                                                                  */


	org MAIN_START


counter:                                                   /* Variablen                                                    */
	fdb  0

result:
	fdb  0


subroutine_StartPSD:
	ldaa $5C00                                              /* "Vin" auf Low setzen                                         */
	rts

subroutine_GetPSD:
	sei
	clr  result
	ldaa #8                                                 /* fuer 8 Schleifendurchlaeufe                                  */
	staa counter
loop:
	ldaa $5E00                                              /* "Vin" auf High setzen                                        */
	ldaa #36                                                /* 100 s warten                                                */
	jsr  wait
	ldaa $5C00                                              /* "Vin" auf Low setzen                                         */
	ldaa #16                                                /* 50 s warten                                                 */
	jsr  wait
	sec                                                     /* damit asl Bit 0 setzt                                        */
	asl  result                                             /* nach links schieben -> naechstes Bit bearbeiten              */
	ldaa $1000                                              /* "Vout" lesen                                                 */
	lsra                                                    /* PA1 selektieren                                              */
	anda #1
	beq  NotSet                                             /* Bit nicht gesetzt -> im 'result'-Vektor nicht loeschen       */
	dec  result                                             /* sonst Bit 0 im 'result'-Vektor loeschen                      */
/* der obige Teil sieht etwas wiederspruechlich aus, das liegt aber daran, dass der Port des 68HC11 invertiert ist, d.h.   */
/* wenn am Sensor eine 1 anliegt, wird eine 0 zurueckgeliefert, es muss aber das entsprechende Bit im 'result' gesetzt     */
/* werden und umgekehrt                                                                                                    */
NotSet:
	ldaa #12                                                /* 40 s warten                                                 */
	jsr  wait
	dec  counter
	beq  end                                                /* schon alle 8 bit gelesen                                     */
	bra  loop                                               /* sonst naechster Schleifendurchlauf                           */
end:
	ldaa $5E00                                              /* "Vin" auf High setzen                                        */
	ldaa #196
	jsr  wait
	cli                                                     /* Interrupts wieder zulassen                                   */
	ldab result                                             /* 'result' in das Low-Byte von Akku D (Rueckgabewert) laden... */
	clra                                                    /* ... und das High-Byte loeschen                               */
	rts

wait:                                                      /* Argument n in Akku A, wartet ( 2.5 * n + 10 ) s             */
	deca
	bne  wait
	rts

