/* Project: Robotik-Praktikum - "Rug Warrior"                                                                              */
/* File   : "SCAN.C"                                                                                                       */
/* Author : Lars Ferner                                                                                                    */
/* Date   : 96-06-12 / 96-02-09                                                                                            */
/* Notes  : Test-Modul                                                                                                     */


/* Global constants                                                                                                        */

float VOLTAGE_ADAPTION = 1.0;
float CLICKS_PER_M = MARKINGS / WHEEL_GIRTH;
float CLICKS_PER_DEGREE = CLICKS_PER_M * M_PI * WHEEL_TRACK / 360.0;
int ADAPTION[] = { 0, 2, 4, 10, 12, 14, 16, 17, 20, 22, 25, 26, 30, 31, 34, 36, 39, 42,
						 44, 46, 52, 54, 56, 60, 64, 67, 72, 76, 78, 80, 86, 88, 92, 98 };
/* int ADAPTION[] = { 0, 2, 4, 10, 14, 16, 17, 19, 22, 24, 26, 30, 35, 39, 43, 48, 52, 56,
						 59, 63, 67, 71, 75, 79, 83, 86, 91, 98 }; */
float DT = 0.2;
int SEPARATOR = '+';
int SECTORMAP = 'S';


/* Global variables                                                                                                        */

int v0 = 0, w0 = 0;


/* Main function                                                                                                           */

void main( void )
/* setzt den Betriebsmodus in Abhaengigkeit von der "persistenten" Variable 'mode'                                         */
{
	/* Local variables  */

	float x, y;

	int i, n = 80, o = 1, d = 0;


	if ( mode == 1 )                                        /* "Ruhe"-Modus                                                 */
	{
		mode = 2;

		motor( LEFT, 0 );
		motor( RIGHT, 0 );
	}
	else if ( mode == 2 )
	{
		mode = 1;                                            /* sollte das Fahren des Quadrates durch ein Reset unterbrochen */
																			  /* werden, zunaechst in den "Ruhe"-Modus verzweigen             */
		init_motors();
		init_velocity();

		printf( "Test...\n" );

		InitSerial();
		
		while ( TRUE )                                       /* fortwaehrendes Uebertragen von Sector Scans                  */
		{
			for ( i = 0; i < 10; i++ )
				SendByte( SEPARATOR );		                    /* 10 Trennzeichen uebertragen                                  */
			SendByte( SECTORMAP );                            /* Kennzeichen fuer Sector Map                                  */
			SendIntByByte( n );                               /* Anzahl der Sektoren                                          */
			SendIntByByte( o );                               /* Scan-Richtung                                                */
			SendIntByByte( d );                               /* Displacement in Grad                                         */
				
			if ( o > 0 )
			{
				motor( RIGHT, 22 );
				motor( LEFT, -22 );
			}
			else
			{
				motor( RIGHT, -22 );
				motor( LEFT, 22 );
			}
			for ( i = 0; i < n; i++ )
			{
				SendIntByByte( psd() );                        /* Entfernungsmesswert                                          */
				
				timer_start( 0, 20L );
				while ( timer_time( 0 ) > 0L );
			}
			move( 0.0, 0.0 );

			o *= -1;
		}

		printf( "Fertig!\n" );

		mode = 2;                                            /* wurde die Aufageb vollstaendig ausgefuehrt, beim naechsten   */
	}                                                       /* Reset gleich wieder in diesen Teil verzweigen                */
	else
		mode = 2;                                            /* Initialisieren der Variable 'mode' beim ersten Durchlauf     */
}


/* Functions (definitions)                                                                                                 */

void move( float v, float w )
/* vw-Schnittstelle: berechnet aus 'v' und 'w' die dem linken/rechten Motor zuzufuehrende Leistung                         */
{
	v *= VOLTAGE_ADAPTION;
	w *= VOLTAGE_ADAPTION;

	if ( v >  0.8 ) v =  0.8;
	if ( v < -0.8 ) v = -0.8;
	if ( w >  180.0 ) w =  180.0;
	if ( w < -180.0 ) w = -180.0;

	v0 = (int) ( 100.0 * CLICKS_PER_M * v * DT );           /* in 100'stel "clicks"                                         */
	w0 = (int) ( 100.0 * CLICKS_PER_DEGREE * w * DT );      /* in 100'stel "clicks"                                         */

	w0 *= 2;
	if ( v0 - w0 < 0 )
		motor( LEFT, -ADAPTION[( w0 - v0 ) / 8] );
	else
		motor( LEFT, ADAPTION[( v0 - w0 ) / 8] );
	motor( RIGHT, ( v0 + w0 ) / 4 );
}

